@@ -1,5 +1,9 @@
-# This file is Copyright (c) 2015-2020 Florent Kermarrec <florent@enjoy-digital.fr>
-# License: BSD
+#
+# This file is part of LitePCIe.
+#
+# Copyright (c) 2015-2020 Florent Kermarrec <florent@enjoy-digital.fr>
+# Copyright (c) 2020 Antmicro <www.antmicro.com>
+# SPDX-License-Identifier: BSD-2-Clause
 
 from migen import *
 from migen.genlib.misc import chooser, displacer
@@ -235,11 +239,11 @@ class LitePCIeDMAReader(Module, AutoCSR):
 
     A MSI IRQ can be generated when a descriptor has been executed.
     """
-    def __init__(self, endpoint, port, table_depth=256):
+    def __init__(self, endpoint, port, with_table=True, table_depth=256):
         self.port   = port
         self.source = stream.Endpoint(dma_layout(endpoint.phy.data_width))
         self.irq    = Signal()
-        self.enable = CSRStorage(description="DMA Reader Control. Write ``1`` to enable DMA Reader.")
+        self.enable = CSRStorage(description="DMA Reader Control. Write ``1`` to enable DMA Reader.", reset=0 if with_table else 1)
 
         # # #
 
@@ -254,18 +258,23 @@ class LitePCIeDMAReader(Module, AutoCSR):
         pending_words_queue   = Signal.like(pending_words)
         pending_words_dequeue = Signal.like(pending_words)
 
-        # Table / Splitter -----------------------------------------------------------------
-        # Descriptors from Table are splitted in descriptors of max_request_size (negotiated at link-up)
-        table    = LitePCIeDMAScatterGather(table_depth)
+        # Table ------------------------------------------------------------------------------------
+        if with_table:
+            self.submodules.table = LitePCIeDMAScatterGather(table_depth)
+        else:
+            self.desc_sink = stream.Endpoint(descriptor_layout()) # Expose a Descriptor sink.
+
+        # Splitter ---------------------------------------------------------------------------------
+        # DMA descriptors need to be splitted in descriptors of max_request_size (negotiated at link-up)
         splitter = LitePCIeDMADescriptorSplitter(max_size=endpoint.phy.max_request_size)
         splitter = ResetInserter()(splitter)
         splitter = BufferizeEndpoints({"source": DIR_SOURCE})(splitter)
-        self.submodules.table    = table
         self.submodules.splitter = splitter
-        self.comb += [
-            splitter.reset.eq(~enable),
-            table.source.connect(splitter.sink)
-        ]
+        self.comb += splitter.reset.eq(~enable)
+        if with_table:
+            self.comb += self.table.source.connect(splitter.sink)
+        else:
+            self.comb += self.desc_sink.connect(splitter.sink)
 
         # Data FIFO --------------------------------------------------------------------------------
         fifo = SyncFIFO(dma_layout(endpoint.phy.data_width), fifo_depth, buffered=True)
@@ -325,7 +334,7 @@ class LitePCIeDMAReader(Module, AutoCSR):
             ),
         ]
         self.sync += [
-            If(~self.enable.storage,
+            If(~enable,
                 pending_words.eq(0)
             ).Else(
                 pending_words.eq(pending_words + pending_words_queue - pending_words_dequeue)
@@ -356,11 +365,11 @@ class LitePCIeDMAWriter(Module, AutoCSR):
 
     A MSI IRQ can be generated when a descriptor has been executed.
     """
-    def __init__(self, endpoint, port, table_depth=256):
+    def __init__(self, endpoint, port, with_table=True, table_depth=256):
         self.port   = port
         self.sink   = sink = stream.Endpoint(dma_layout(endpoint.phy.data_width))
         self.irq    = Signal()
-        self.enable = CSRStorage(description="DMA Writer Control. Write ``1`` to enable DMA Writer.")
+        self.enable = CSRStorage(description="DMA Writer Control. Write ``1`` to enable DMA Writer.", reset=0 if with_table else 1)
 
         # # #
 
@@ -372,18 +381,23 @@ class LitePCIeDMAWriter(Module, AutoCSR):
         max_words_per_request = max_payload_size//(endpoint.phy.data_width//8)
         fifo_depth            = 4*max_words_per_request
 
-        # Table/Splitter ---------------------------------------------------------------------------
-        # Descriptors from table are splitted in descriptors of max_payload_size (negotiated at link-up)
-        table    = LitePCIeDMAScatterGather(table_depth)
+        # Table ------------------------------------------------------------------------------------
+        if with_table:
+            self.submodules.table = LitePCIeDMAScatterGather(table_depth)
+        else:
+            self.desc_sink = stream.Endpoint(descriptor_layout()) # Expose a Descriptor sink.
+
+        # Splitter ---------------------------------------------------------------------------------
+        # DMA descriptors need to be splitted in descriptors of max_request_size (negotiated at link-up)
         splitter = LitePCIeDMADescriptorSplitter(max_size=endpoint.phy.max_payload_size)
         splitter = ResetInserter()(splitter)
         splitter = BufferizeEndpoints({"source": DIR_SOURCE})(splitter)
-        self.submodules.table    = table
         self.submodules.splitter = splitter
-        self.comb += [
-            splitter.reset.eq(~enable),
-            table.source.connect(splitter.sink)
-        ]
+        self.comb += splitter.reset.eq(~enable)
+        if with_table:
+            self.comb += self.table.source.connect(splitter.sink)
+        else:
+            self.comb += self.desc_sink.connect(splitter.sink)
 
         # Data FIFO --------------------------------------------------------------------------------
         fifo = SyncFIFOBuffered(endpoint.phy.data_width + 1, fifo_depth)
