@@ -1,5 +1,8 @@
-# This file is Copyright (c) 2015-2020 Florent Kermarrec <florent@enjoy-digital.fr>
-# License: BSD
+#
+# This file is part of LitePCIe.
+#
+# Copyright (c) 2015-2020 Florent Kermarrec <florent@enjoy-digital.fr>
+# SPDX-License-Identifier: BSD-2-Clause
 
 import os
 
@@ -9,67 +12,43 @@ from migen.genlib.cdc import MultiReg
 from litex.soc.interconnect.csr import *
 
 from litepcie.common import *
-
-# AXISRX128BAligner --------------------------------------------------------------------------------
-
-class AXISRX128BAligner(Module):
-    def __init__(self):
-        self.sink   = sink   = stream.Endpoint(phy_layout(128))
-        self.source = source = stream.Endpoint(phy_layout(128))
-        self.first_dword = Signal(2)
-
-        # # #
-
-        dat_last = Signal(64, reset_less=True)
-        be_last  = Signal(8, reset_less=True)
-        self.sync += [
-            If(sink.valid & sink.ready,
-                dat_last.eq(sink.dat[64:]),
-                be_last.eq( sink.be[8:]),
-            )
-        ]
-
-        self.submodules.fsm = fsm = FSM(reset_state="ALIGNED")
-        fsm.act("ALIGNED",
-            sink.connect(source, omit={"first"}),
-            # If "first" on DWORD2 and "last" on the same cycle, switch to UNALIGNED.
-            If(sink.valid & sink.last & sink.first & (self.first_dword == 2),
-                source.be[8:].eq(0),
-                If(source.ready,
-                    NextState("UNALIGNED")
-                )
-            )
-        )
-        fsm.act("UNALIGNED",
-            sink.connect(source, omit={"first", "dat", "be"}),
-            source.dat.eq(Cat(dat_last, sink.dat)),
-            source.be.eq( Cat(be_last,  sink.be)),
-            # If "last" and not "first" on the same cycle, switch to ALIGNED.
-            If(sink.valid & sink.last & ~sink.first,
-                source.be[8:].eq(0),
-                If(source.ready,
-                    NextState("ALIGNED")
-                )
-            )
-        )
+from litepcie.phy.common import *
 
 # S7PCIEPHY ----------------------------------------------------------------------------------------
 
 class S7PCIEPHY(Module, AutoCSR):
-    def __init__(self, platform, pads, data_width=64, bar0_size=1*MB, cd="sys", pcie_data_width=64):
+    def __init__(self, platform, pads, data_width=64, bar0_size=1*MB, cd="sys", pcie_data_width=None):
         # Streams ----------------------------------------------------------------------------------
         self.sink   = stream.Endpoint(phy_layout(data_width))
         self.source = stream.Endpoint(phy_layout(data_width))
         self.msi    = stream.Endpoint(msi_layout())
 
         # Registers --------------------------------------------------------------------------------
-        self._link_up           = CSRStatus(description="Link Up Status. ``1``: Link is Up.")
+        self._link_status = CSRStatus(fields=[
+            CSRField("status", size=1, values=[
+                ("``0b0``", "Link Down."),
+                ("``0b1``", "Link Up."),
+            ]),
+            CSRField("rate", size=1, values=[
+                ("``0b0``", "2.5 Gb/s."),
+                ("``0b1``", "5.0 Gb/s."),
+            ]),
+            CSRField("width", size=2, values=[
+                ("``0b00``", "1-Lane link."),
+                ("``0b01``", "2-Lane link."),
+                ("``0b10``", "4-Lane link."),
+                ("``0b11``", "8-Lane link."),
+            ]),
+            CSRField("ltssm", size=6, description="LTSSM State"),
+        ])
         self._msi_enable        = CSRStatus(description="MSI Enable Status. ``1``: MSI is enabled.")
+        self._msix_enable       = CSRStatus(description="MSI-X Enable Status. ``1``: MSI-X is enabled.")
         self._bus_master_enable = CSRStatus(description="Bus Mastering Status. ``1``: Bus Mastering enabled.")
         self._max_request_size  = CSRStatus(16, description="Negiotiated Max Request Size (in bytes).")
         self._max_payload_size  = CSRStatus(16, description="Negiotiated Max Payload Size (in bytes).")
 
         # Parameters/Locals ------------------------------------------------------------------------
+        if pcie_data_width is None: pcie_data_width = data_width
         self.platform         = platform
         self.data_width       = data_width
         self.pcie_data_width  = pcie_data_width
@@ -90,10 +69,13 @@ class S7PCIEPHY(Module, AutoCSR):
         assert data_width      in [64, 128]
         assert pcie_data_width in [64, 128]
 
-        # Clocking ---------------------------------------------------------------------------------
-        pcie_refclk = Signal()
+        # Clocking / Reset -------------------------------------------------------------------------
+        self.pcie_refclk = pcie_refclk = Signal()
+        self.pcie_rst_n  = pcie_rst_n  = Signal(reset=1)
+        if hasattr(pads, "rst_n"):
+            self.comb += pcie_rst_n.eq(pads.rst_n)
         self.specials += Instance("IBUFDS_GTE2",
-            i_CEB = 0,
+            i_CEB = ~pcie_rst_n,
             i_I   = pads.clk_p,
             i_IB  = pads.clk_n,
             o_O   = pcie_refclk
@@ -101,55 +83,22 @@ class S7PCIEPHY(Module, AutoCSR):
         platform.add_period_constraint(pads.clk_p, 1e9/100e6)
         self.clock_domains.cd_pcie = ClockDomain()
         pcie_clk_freq = max(125e6, nlanes*62.5e6*64/pcie_data_width)
-        platform.add_period_constraint(self.cd_pcie.clk, 1e9/pcie_clk_freq)
 
-        # TX (FPGA --> HOST) CDC / Data Width Convertion -------------------------------------------
-        if (cd == "pcie") and (data_width == pcie_data_width):
-            s_axis_tx = self.sink
-        else:
-            tx_pipe_valid = stream.PipeValid(phy_layout(data_width))
-            tx_pipe_valid = ClockDomainsRenamer(cd)(tx_pipe_valid)
-            tx_cdc        = stream.AsyncFIFO(phy_layout(data_width), 4)
-            tx_cdc        = ClockDomainsRenamer({"write": cd, "read": "pcie"})(tx_cdc)
-            tx_converter  = stream.StrideConverter(phy_layout(data_width), phy_layout(pcie_data_width))
-            tx_converter  = ClockDomainsRenamer("pcie")(tx_converter)
-            tx_pipe_ready = stream.PipeValid(phy_layout(pcie_data_width))
-            tx_pipe_ready = ClockDomainsRenamer("pcie")(tx_pipe_ready)
-            self.submodules += tx_pipe_valid, tx_cdc, tx_converter, tx_pipe_ready
-            self.comb += [
-                self.sink.connect(tx_pipe_valid.sink),
-                tx_pipe_valid.source.connect(tx_cdc.sink),
-                tx_cdc.source.connect(tx_converter.sink),
-                tx_converter.source.connect(tx_pipe_ready.sink)
-            ]
-            s_axis_tx = tx_pipe_ready.source
+        # TX (FPGA --> HOST) CDC / Data Width Conversion -------------------------------------------
+        self.submodules.tx_datapath = PHYTXDatapath(
+            core_data_width = data_width,
+            pcie_data_width = pcie_data_width,
+            clock_domain    = cd)
+        self.comb += self.sink.connect(self.tx_datapath.sink)
+        s_axis_tx = self.tx_datapath.source
 
-        # RX (HOST --> FPGA) CDC / Data Width Convertion -------------------------------------------
-        if (cd == "pcie") and (data_width == pcie_data_width):
-            m_axis_rx = self.source
-        else:
-            rx_pipe_ready = stream.PipeReady(phy_layout(pcie_data_width))
-            rx_pipe_ready = ClockDomainsRenamer("pcie")(rx_pipe_ready)
-            rx_converter  = stream.StrideConverter(phy_layout(pcie_data_width), phy_layout(data_width))
-            rx_converter  = ClockDomainsRenamer("pcie")(rx_converter)
-            rx_cdc        = stream.AsyncFIFO(phy_layout(data_width), 4)
-            rx_cdc        = ClockDomainsRenamer({"write": "pcie", "read": cd})(rx_cdc)
-            rx_pipe_valid = stream.PipeValid(phy_layout(data_width))
-            rx_pipe_valid = ClockDomainsRenamer(cd)(rx_pipe_valid)
-            self.submodules += rx_pipe_ready, rx_converter, rx_pipe_valid, rx_cdc
-            self.comb += [
-                rx_pipe_ready.source.connect(rx_converter.sink),
-                rx_converter.source.connect(rx_cdc.sink),
-                rx_cdc.source.connect(rx_pipe_valid.sink),
-                rx_pipe_valid.source.connect(self.source),
-            ]
-            m_axis_rx = rx_pipe_ready.sink
-        if pcie_data_width == 128:
-            rx_aligner = AXISRX128BAligner()
-            rx_aligner = ClockDomainsRenamer("pcie")(rx_aligner)
-            self.submodules += rx_aligner
-            self.comb += rx_aligner.source.connect(m_axis_rx)
-            m_axis_rx = rx_aligner.sink
+        # RX (HOST --> FPGA) CDC / Data Width Conversion -------------------------------------------
+        self.submodules.rx_datapath = PHYRXDatapath(
+            core_data_width = data_width,
+            pcie_data_width = pcie_data_width,
+            clock_domain    = cd)
+        m_axis_rx = self.rx_datapath.sink
+        self.comb += self.rx_datapath.source.connect(self.source)
 
         # MSI CDC (FPGA --> HOST) ------------------------------------------------------------------
         if cd == "pcie":
@@ -170,8 +119,6 @@ class S7PCIEPHY(Module, AutoCSR):
                 value = min(value*2, max_size)
             return Case(command, cases)
 
-        link_up         = Signal()
-        msi_enable      = Signal()
         bus_number      = Signal(8)
         device_number   = Signal(5)
         function_number = Signal(3)
@@ -183,9 +130,7 @@ class S7PCIEPHY(Module, AutoCSR):
             self.id.eq(Cat(function_number, device_number, bus_number))
         ]
         self.specials += [
-            MultiReg(link_up, self._link_up.status),
-            MultiReg(command[2], self._bus_master_enable.status),
-            MultiReg(msi_enable, self._msi_enable.status),
+            MultiReg(command[2],  self._bus_master_enable.status),
             MultiReg(self.max_request_size, self._max_request_size.status),
             MultiReg(self.max_payload_size, self._max_payload_size.status)
         ]
@@ -194,6 +139,7 @@ class S7PCIEPHY(Module, AutoCSR):
         class Open(Signal): pass
         m_axis_rx_tlast = Signal()
         m_axis_rx_tuser = Signal(32)
+        self.pcie_gt_device  = {"xc7a": "GTP", "xc7k": "GTX", "xc7v": "GTX"}[platform.device[:4]]
         self.pcie_phy_params = dict(
             # Parameters ---------------------------------------------------------------------------
             p_LINK_CAP_MAX_LINK_WIDTH                    = nlanes,
@@ -202,14 +148,11 @@ class S7PCIEPHY(Module, AutoCSR):
             p_PCIE_REFCLK_FREQ                           = 0, # 100MHz refclk
             p_PCIE_USERCLK1_FREQ                         = 3 if nlanes <= 2 else 4,
             p_PCIE_USERCLK2_FREQ                         = 3 if (pcie_clk_freq == 125e6) else 4,
-            p_PCIE_GT_DEVICE                             = {"xc7a": "GTP",
-                                                            "xc7k": "GTX",
-                                                            "xc7v": "GTX"}[platform.device[:4]],
-            p_PCIE_USE_MODE                              = "1.0",
+            p_PCIE_GT_DEVICE                             = self.pcie_gt_device,
 
             # PCI Express Interface ----------------------------------------------------------------
             i_sys_clk                                    = pcie_refclk,
-            i_sys_rst_n                                  = 1 if not hasattr(pads, "rst_n") else pads.rst_n,
+            i_sys_rst_n                                  = pcie_rst_n,
             # TX
             o_pci_exp_txp                                = pads.tx_p,
             o_pci_exp_txn                                = pads.tx_n,
@@ -233,7 +176,7 @@ class S7PCIEPHY(Module, AutoCSR):
             # Common
             o_user_clk_out                               = ClockSignal("pcie"),
             o_user_reset_out                             = ResetSignal("pcie"),
-            o_user_lnk_up                                = link_up,
+            o_user_lnk_up                                = self._link_status.fields.status,
             o_user_app_rdy                               = Open(),
 
             # TX
@@ -325,8 +268,8 @@ class S7PCIEPHY(Module, AutoCSR):
             i_cfg_interrupt_di                           = cfg_msi.dat,
             o_cfg_interrupt_do                           = Open(),
             o_cfg_interrupt_mmenable                     = Open(),
-            o_cfg_interrupt_msienable                    = msi_enable,
-            o_cfg_interrupt_msixenable                   = Open(),
+            o_cfg_interrupt_msienable                    = self._msi_enable.status,
+            o_cfg_interrupt_msixenable                   = self._msix_enable.status,
             o_cfg_interrupt_msixfm                       = Open(),
             i_cfg_interrupt_stat                         = 0,
             i_cfg_pciecap_interrupt_msgnum               = 0,
@@ -390,9 +333,9 @@ class S7PCIEPHY(Module, AutoCSR):
             i_pl_directed_link_speed                     = 0,
             i_pl_directed_link_auton                     = 0,
             i_pl_upstream_prefer_deemph                  = 1,
-            o_pl_sel_lnk_rate                            = Open(),
-            o_pl_sel_lnk_width                           = Open(),
-            o_pl_ltssm_state                             = Open(),
+            o_pl_sel_lnk_rate                            = self._link_status.fields.rate,
+            o_pl_sel_lnk_width                           = self._link_status.fields.width,
+            o_pl_ltssm_state                             = self._link_status.fields.ltssm,
             o_pl_lane_reversal_mode                      = Open(),
             o_pl_phy_lnk_up                              = Open(),
             o_pl_tx_pm_state                             = Open(),
@@ -421,7 +364,7 @@ class S7PCIEPHY(Module, AutoCSR):
             self.comb += [
                 m_axis_rx.first.eq(rx_is_sof[-1]),
                 m_axis_rx.last.eq( rx_is_eof[-1]),
-                If(rx_is_sof == 0b11000, rx_aligner.first_dword.eq(2)),
+                If(rx_is_sof == 0b11000, self.rx_datapath.aligner.first_dword.eq(2)),
             ]
         else:
             self.comb += [
@@ -431,9 +374,15 @@ class S7PCIEPHY(Module, AutoCSR):
 
     # Hard IP sources ------------------------------------------------------------------------------
     def add_sources(self, platform, phy_path, phy_filename):
-        platform.add_ip(os.path.join(phy_path, phy_filename))
         platform.add_source(os.path.join(phy_path, "pcie_pipe_clock.v"))
-        platform.add_source(os.path.join(phy_path, "pcie_s7_x{}_support.v".format(self.nlanes)))
+        platform.add_source(os.path.join(phy_path, "pcie_s7_support.v"))
+        platform.add_ip(os.path.join(phy_path, phy_filename))
+        # Reset LOC constraints on GTPE2_COMMON and BRAM36 from .xci (we only want to keep Timing constraints).
+        if self.pcie_gt_device == "GTP":
+            platform.toolchain.pre_placement_commands.append("reset_property LOC [get_cells -hierarchical -filter {{NAME=~pcie_support/*gtp_common.gtpe2_common_i}}]")
+        else:
+            platform.toolchain.pre_placement_commands.append("reset_property LOC [get_cells -hierarchical -filter {{NAME=~pcie_support/*gtx_common.gtxe2_common_i}}]")
+        platform.toolchain.pre_placement_commands.append("reset_property LOC [get_cells -hierarchical -filter {{NAME=~pcie_support/*genblk*.bram36_tdp_bl.bram36_tdp_bl}}]")
 
     # External Hard IP -----------------------------------------------------------------------------
     def use_external_hard_ip(self, hard_ip_path, hard_ip_filename):
@@ -443,10 +392,9 @@ class S7PCIEPHY(Module, AutoCSR):
     # Finalize -------------------------------------------------------------------------------------
     def do_finalize(self):
         if not self.external_hard_ip:
-            phy_path     = "xilinx_s7_x{}".format(self.nlanes)
-            phy_filename = "pcie_s7_x{}.xci".format(self.nlanes)
+            phy_path     = "xilinx_s7_gen2_x{}".format(self.nlanes)
             self.add_sources(self.platform,
                 phy_path     = os.path.join(os.path.abspath(os.path.dirname(__file__)), phy_path),
-                phy_filename = "pcie_s7_x{}.xci".format(self.nlanes)
+                phy_filename = "pcie_s7.xci"
             )
         self.specials += Instance("pcie_support", **self.pcie_phy_params)
